////////////////////////////////
//Font class implementation.
//Version: 110113_01
////////////////////////////////

#include "font.h"

struct VertexPos
{
    XMFLOAT3 pos;
    XMFLOAT2 tex0;
};


void FontClass::Init(ID3D11Device* d3dDevice)
{
	ID3DBlob* vsBuffer = 0;

    CompileD3DShader( L"Shaders/font.fx", "VS_Main", "vs_4_0", &vsBuffer );

    HRESULT d3dResult;

    d3dResult = d3dDevice->CreateVertexShader( vsBuffer->GetBufferPointer( ), vsBuffer->GetBufferSize( ), 0, &solidColorVS_ );


    D3D11_INPUT_ELEMENT_DESC solidColorLayout[] =
    {
        { "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D11_INPUT_PER_VERTEX_DATA, 0 },
        { "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12, D3D11_INPUT_PER_VERTEX_DATA, 0 }
    };

    unsigned int totalLayoutElements = ARRAYSIZE( solidColorLayout );

    d3dDevice->CreateInputLayout( solidColorLayout, totalLayoutElements, vsBuffer->GetBufferPointer( ), vsBuffer->GetBufferSize( ), &inputLayout_ );

    vsBuffer->Release( );


    ID3DBlob* psBuffer = 0;

    CompileD3DShader( L"Shaders/font.fx", "PS_Main", "ps_4_0", &psBuffer );


    d3dResult = d3dDevice->CreatePixelShader( psBuffer->GetBufferPointer( ), psBuffer->GetBufferSize( ), 0, &solidColorPS_ );

    psBuffer->Release( );


	D3DX11CreateShaderResourceViewFromFile( d3dDevice, L"Data/font.bmp", 0, 0, &colorMap_, 0 );


    D3D11_SAMPLER_DESC colorMapDesc;
    ZeroMemory( &colorMapDesc, sizeof( colorMapDesc ) );
    colorMapDesc.AddressU = D3D11_TEXTURE_ADDRESS_WRAP;
    colorMapDesc.AddressV = D3D11_TEXTURE_ADDRESS_WRAP;
    colorMapDesc.AddressW = D3D11_TEXTURE_ADDRESS_WRAP;
    colorMapDesc.ComparisonFunc = D3D11_COMPARISON_NEVER;
    colorMapDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
    colorMapDesc.MaxLOD = D3D11_FLOAT32_MAX;

    d3dDevice->CreateSamplerState( &colorMapDesc, &colorMapSampler_ );

    D3D11_BUFFER_DESC vertexDesc;
    ZeroMemory( &vertexDesc, sizeof( vertexDesc ) );
    vertexDesc.Usage = D3D11_USAGE_DYNAMIC;
    vertexDesc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
    vertexDesc.BindFlags = D3D11_BIND_VERTEX_BUFFER;

    const int sizeOfSprite = sizeof( VertexPos ) * 6;
    const int maxLetters = 24;

    vertexDesc.ByteWidth = sizeOfSprite * maxLetters;

    d3dResult = d3dDevice->CreateBuffer( &vertexDesc, 0, &vertexBuffer_ );
}


void FontClass::Render(ID3D11DeviceContext* d3dContext, IDXGISwapChain* swapChain, ID3D11RenderTargetView* backBufferTarget, char* string, float posX, float posY)
{
	//float clearColor[4] = { 0.2f, 0.22f, 0.24f, 1.0f };
    //d3dContext->ClearRenderTargetView( backBufferTarget, clearColor );

    unsigned int stride = sizeof( VertexPos );
    unsigned int offset = 0;

    d3dContext->IASetInputLayout( inputLayout_ );
    d3dContext->IASetVertexBuffers( 0, 1, &vertexBuffer_, &stride, &offset );
    d3dContext->IASetPrimitiveTopology( D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

    d3dContext->VSSetShader( solidColorVS_, 0, 0 );
    d3dContext->PSSetShader( solidColorPS_, 0, 0 );
    d3dContext->PSSetShaderResources( 0, 1, &colorMap_ );
    d3dContext->PSSetSamplers( 0, 1, &colorMapSampler_ );
    
    DrawString(d3dContext, string, posX, posY);

    swapChain->Present( 0, 0 );
}

void FontClass::Release()
{
	if( colorMapSampler_ ) colorMapSampler_->Release( );
    if( colorMap_ ) colorMap_->Release( );
    if( solidColorVS_ ) solidColorVS_->Release( );
    if( solidColorPS_ ) solidColorPS_->Release( );
    if( inputLayout_ ) inputLayout_->Release( );
    if( vertexBuffer_ ) vertexBuffer_->Release( );

    colorMapSampler_ = 0;
    colorMap_ = 0;
    solidColorVS_ = 0;
    solidColorPS_ = 0;
    inputLayout_ = 0;
    vertexBuffer_ = 0;
}

void FontClass::DrawString(ID3D11DeviceContext* d3dContext, char* message, float startX, float startY)
{
    const int sizeOfSprite = sizeof( VertexPos ) * 6;

    const int maxLetters = 128;

    int length = strlen( message );

    if( length > maxLetters )
        length = maxLetters;

    float charWidth = 32.0f / 800.0f;

    float charHeight = 32.0f / 640.0f;
    
    float texelWidth = 32.0f / 864.0f; //change this as needed

    const int verticesPerLetter = 6;

    D3D11_MAPPED_SUBRESOURCE mapResource;
    HRESULT d3dResult = d3dContext->Map( vertexBuffer_, 0, D3D11_MAP_WRITE_DISCARD, 0, &mapResource );

    // Point to our vertex buffer's internal data.
    VertexPos *spritePtr = ( VertexPos* )mapResource.pData;

    const int indexA = static_cast<char>( 'A' );
    const int indexZ = static_cast<char>( 'Z' );

    for( int i = 0; i < length; ++i )
    {
        float thisStartX = startX + ( charWidth * static_cast<float>( i ) );
        float thisEndX = thisStartX + charWidth;
        float thisEndY = startY + charHeight;

        spritePtr[0].pos = XMFLOAT3( thisEndX,   thisEndY, 1.0f );
        spritePtr[1].pos = XMFLOAT3( thisEndX,   startY,   1.0f );
        spritePtr[2].pos = XMFLOAT3( thisStartX, startY,   1.0f );
        spritePtr[3].pos = XMFLOAT3( thisStartX, startY,   1.0f );
        spritePtr[4].pos = XMFLOAT3( thisStartX, thisEndY, 1.0f );
        spritePtr[5].pos = XMFLOAT3( thisEndX,   thisEndY, 1.0f );

        int texLookup = 0;
        int letter = static_cast<char>( message[i] );

        if( letter < indexA || letter > indexZ )
        {
            // Grab one index past Z, which is a blank space in the texture.
            texLookup = ( indexZ - indexA ) + 1;
        }
        else
        {
            // A = 0, B = 1, Z = 25, etc.
            texLookup = ( letter - indexA );
        }

        float tuStart = 0.0f + ( texelWidth * static_cast<float>( texLookup ) );
        float tuEnd = tuStart + texelWidth;

        spritePtr[0].tex0 = XMFLOAT2( tuEnd, 0.0f );
        spritePtr[1].tex0 = XMFLOAT2( tuEnd, 1.0f );
        spritePtr[2].tex0 = XMFLOAT2( tuStart, 1.0f );
        spritePtr[3].tex0 = XMFLOAT2( tuStart, 1.0f );
        spritePtr[4].tex0 = XMFLOAT2( tuStart, 0.0f );
        spritePtr[5].tex0 = XMFLOAT2( tuEnd, 0.0f );

        spritePtr += 6;
    }

    d3dContext->Unmap( vertexBuffer_, 0 );
    d3dContext->Draw( 6 * length, 0 );
}