///////////////////////////////////////////////////////////
// Quaternion rotation based camera class implementation.
// Version: 100113_01
///////////////////////////////////////////////////////////

#include "quatCamera.h"

//debug
#include <iostream>
using namespace std;

void QuaternionCamera::InitCamera()
{
	m_vEyePt	= D3DXVECTOR3(0.0f, 1.0f, -15.0f);
	m_vLookAtPt = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	m_vUp		= D3DXVECTOR3(0.0f, 1.0f, 0.0f);
	m_vRight    = D3DXVECTOR3(1.0f, 0.0f, 1.0f);

	m_speed			= 0.00001f;
	m_turnSpeed		= 0.0025f;
	m_rollSpeed		= 0.005f;
	m_pitchSpeed	= 0.005f;

	//Start the plane in ground landing mode
	m_vEyePt.y = 0.05f; //0.05f is the gorund level

	//pitch, yaw and roll coefficients
	pitch	= 0.0f;
	roll	= 0.0f;
	yaw		= 0.70f;

	//Plane dynamics calculation coefficients
	m_planeMass = 500.0f; //500 kg

	D3DXQuaternionIdentity(&m_rotation);
	D3DXMatrixIdentity(&m_viewMatrix);
}

void QuaternionCamera::UpdateCamera()
{
	//reconstruct the view matrix here to take into account the rotation, translation and rotation based translation
	D3DXMATRIX matTranslation;
	D3DXMatrixIdentity(&matTranslation);
	D3DXMatrixTranslation(  &matTranslation , -m_vEyePt.x , -m_vEyePt.y , -m_vEyePt.z );

	D3DXMATRIX matRotation;
	D3DXMatrixIdentity(&matRotation);

	D3DXQuaternionNormalize(&m_rotation, &m_rotation);
	D3DXMatrixRotationQuaternion( &matRotation,  &D3DXQUATERNION(-m_rotation.x, -m_rotation.y,  -m_rotation.z, m_rotation.w));

	D3DXMatrixMultiply(&m_viewMatrix, &matTranslation, &matRotation);
}

void QuaternionCamera::PitchUp()
{
	pitch += m_pitchSpeed;
}

void QuaternionCamera::PitchDown()
{
	pitch -= m_pitchSpeed;
}

void QuaternionCamera::RollLeft()
{
	roll +=  m_rollSpeed;
}

void QuaternionCamera::RollRight()
{
	roll -=  m_rollSpeed;
}

void QuaternionCamera::YawLeft()
{
	yaw -= m_turnSpeed;
}

void QuaternionCamera::YawRight()
{
	yaw += m_turnSpeed;
}

void QuaternionCamera::Thrust(float deltaTime)
{
	D3DXVECTOR3	 vDirection;

	D3DXVec3Normalize(&vDirection,&(m_vLookAtPt - m_vEyePt));

	D3DXMATRIX matRotation;
	D3DXMatrixIdentity(&matRotation);

	D3DXQuaternionNormalize(&m_rotation, &m_rotation);
	D3DXMatrixRotationQuaternion( &matRotation,  &D3DXQUATERNION(m_rotation.x, m_rotation.y,  m_rotation.z, m_rotation.w));

	D3DXVec3TransformCoord(&vDirection,&vDirection,&(matRotation));

	m_vEyePt += vDirection * m_speed ;
	m_vLookAtPt += vDirection * (m_speed * deltaTime);

}

void QuaternionCamera::IncreaseThrust()
{
	if(m_speed <= 0.27f)
	{
		m_speed += (m_planeMass * 0.0098f) * 0.000015f;
	}

}

void QuaternionCamera::DecreaseThrust()
{
	if(m_speed > 0.0f)
	{
		m_speed -= (m_planeMass * 0.0098f) * 0.000035f;
	}
}

void QuaternionCamera::ApplyFriction(float deltaTime)
{
	m_speed -= (m_planeMass * (0.0098f + 0.0055f)) * deltaTime;
}

void QuaternionCamera::Flaps(float deltaTime)
{
	m_speed -= 0.5f * deltaTime;
}

void QuaternionCamera::Stall(float deltaTime)
{
	m_speed -= (0.5f + 0.0098f) * deltaTime;
}

void QuaternionCamera::StepSimulation(float deltaTime)
{
	//Always keep the plane above ground level
	if(m_speed < 0.0f)
	{
		m_speed = 0.001f;
	}

	//Apply gravity (and keep the plane above ground level)
	if(m_vEyePt.y > 0.05f)
	{
		m_vEyePt.y -= ((m_planeMass * 0.98f)) * deltaTime;
	}
	if(m_vEyePt.y < 0.0f && m_vEyePt.y < 0.05f)
	{
		//if plane is in contact with runway, apply friction coefficient
		ApplyFriction(deltaTime);
		m_vEyePt.y += 0.01f; //this seems to work ok...
	}
	
	//Apply linear velocity
	Thrust(deltaTime);
	
	//Apply angular velocity (only if plane is above a certain altitude)
	if(m_vEyePt.y > 0.05f)
	{
		if(pitch < 0.0f)
		{
			pitch -= ((m_planeMass * 0.0098f) + m_speed) * deltaTime;
		}
		if(pitch > 0.0f)
		{
			pitch += ((m_planeMass * 0.0098f) + m_speed) * deltaTime;
		}

		if(roll < 0.0f)
		{
			roll -= ((m_planeMass * 0.0098f) + m_speed) * deltaTime;
		}
		if(roll > 0.0f)
		{
			roll += ((m_planeMass * 0.0098f) + m_speed) * deltaTime;
		}
	}

	D3DXQuaternionRotationYawPitchRoll(&m_rotation, yaw, pitch, roll);
}
