/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Quaternion rotation based camera class.
// Flightsim specific, using the implicit Euler integration method for flight dynamics integration calculation. 
//
// Based on the flightsim physics code from "Physics for Game Programmers" [Boer 2001].
//
// Written by Vladeta Stojanovic.
// Version: 100113_01
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _QUATCAMERA_H_
#define _QUATCAMERA_H_

#include <d3dx10math.h>

class QuaternionCamera
{
//Member functions
public:
	QuaternionCamera(){}
	~QuaternionCamera(){}

	void InitCamera();
	void UpdateCamera();

	inline void GetViewMatrix(D3DXMATRIX& viewMatrix) {viewMatrix = m_viewMatrix;} 

	////////////////////////
	//Flight sim specific
	////////////////////////
	void PitchUp();
	void PitchDown();
	void YawLeft();
	void YawRight();
	void RollLeft();
	void RollRight();

	void Thrust(float deltaTime);
	void Stall(float deltaTime);
	void IncreaseThrust();
	void DecreaseThrust();

	////////////////////////////////////
	// Time step physics simulation
	////////////////////////////////////
	void StepSimulation(float deltaTime);

private:

	inline D3DXVECTOR3* TransformVector(D3DXQUATERNION *pOrientation, D3DXVECTOR3 *pAxis)
	{
		D3DVECTOR vNewAxis;
		D3DXMATRIX matRotation;

		D3DXMatrixRotationQuaternion(&matRotation, pOrientation); 

		vNewAxis.x = pAxis->x * matRotation._11 + pAxis->y * matRotation._21 + pAxis->z * matRotation._31 + matRotation._41; 
		vNewAxis.y = pAxis->x * matRotation._12 + pAxis->y * matRotation._22 + pAxis->z * matRotation._32 + matRotation._42;
		vNewAxis.z = pAxis->x * matRotation._13 + pAxis->y * matRotation._23 + pAxis->z * matRotation._33 + matRotation._43;

		memcpy(pAxis, &vNewAxis, sizeof(vNewAxis)); //errr....

		return(pAxis);
	}

	////////////////////////
	//Flight sim specific
	////////////////////////
public:
	void ApplyFriction(float deltaTime);
	void Flaps(float deltaTime);

//Data members
public:
	float m_speed;
	float m_turnSpeed;
	float m_rollSpeed;
	float m_pitchSpeed;

	//Plane dynamics specific (all in international SI units)
	float pitch;
	float roll;
	float yaw;
	float m_planeMass;

	D3DXVECTOR3		m_vEyePt;

private:
	D3DXVECTOR3		m_vLookAtPt, m_vUp, m_vRight; //eye = position
	D3DXVECTOR3		vDirection;
	D3DXQUATERNION	m_rotation;
	D3DXMATRIX		m_viewMatrix;
	D3DXMATRIX		matRotation;
	D3DXMATRIX		matTranslation;
};

#endif